from __future__ import annotations

from django import template

from .tags import asset
from .tags import bird
from .tags import prop
from .tags import slot
from .tags import var

register = template.Library()


register.tag(asset.AssetTag.CSS.value, asset.do_asset)
register.tag(asset.AssetTag.JS.value, asset.do_asset)
register.tag(bird.TAG, bird.do_bird)
register.tag(prop.TAG, prop.do_prop)
register.tag(slot.TAG, slot.do_slot)
register.tag(var.TAG, var.do_var)
register.tag(var.END_TAG, var.do_end_var)



from __future__ import annotations

from typing import final

from django.apps import AppConfig

from ._typing import override


@final
class DjangoBirdAppConfig(AppConfig):
    label = "django_bird"
    name = "django_bird"
    verbose_name = "Bird"

    @override
    def ready(self):
        from django_bird.components import components
        from django_bird.conf import app_settings
        from django_bird.plugins import pm
        from django_bird.staticfiles import asset_types

        app_settings.autoconfigure()
        pm.hook.register_asset_types(register_type=asset_types.register_type)
        components.discover_components()



from __future__ import annotations

from collections.abc import Callable
from collections.abc import Iterable
from pathlib import Path
from typing import TYPE_CHECKING

from pluggy import HookspecMarker

if TYPE_CHECKING:
    from django_bird.staticfiles import Asset
    from django_bird.staticfiles import AssetType

hookspec = HookspecMarker("django_bird")


@hookspec
def collect_component_assets(template_path: Path) -> Iterable[Asset]:
    """Collect all assets associated with a component.

    This hook is called for each component template to gather its associated static assets.
    Implementations should scan for and return any CSS, JavaScript or other static files
    that belong to the component and return a list/set/other iterable of `Asset` objects.
    """


@hookspec
def get_template_directories() -> list[Path]:
    """Return a list of all directories containing templates for a project.

    This hook allows plugins to provide additional template directories beyond the default
    Django template directories. Implementations should return a list of Path objects
    pointing to directories that contain Django templates.

    The template directories returned by this hook will be used by django-bird to discover
    components and their associated templates.
    """


@hookspec
def register_asset_types(register_type: Callable[[AssetType], None]) -> None:
    """Register a new type of asset.

    This hook allows plugins to register additional asset types beyond the default CSS
    and JS types. Each asset type defines how static assets should be rendered in HTML,
    what file extension it uses, and what django-bird asset templatetag it should be
    rendered with.
    """
